﻿using System;
using System.IO;
using System.Threading;
using System.Collections.Generic;
using System.Reflection;

#pragma warning disable 0618

namespace System.Threading {

    public class MyMonitor {

        readonly static string CoverTag = "CC_COVER_TAG_CC";

        const int ByteSize = 1024*1024/2;     // Lets use 1M total.
        static long[] mHitCount = new long[ByteSize / sizeof(long)];
        static long[] mContentCount = new long[ByteSize / sizeof(long)];
        static MyMonitor sFoo = new MyMonitor();

        static DateTime sTime = DateTime.Now;

        public static string GetCoverFile() {
            return "cc.cov." + CoverTag + "." + Environment.MachineName + ".txt";
        }

        ~MyMonitor() {
            try {
                Dump(GetCoverFile());
            } catch (Exception e) {
                Console.Error.WriteLine("~MyMonitor:Catch(" + e.Message + ")");
                Console.Error.WriteLine(e);
            }
        }

        public static void Clear() {
            for (int i=0; i < mHitCount.Length; ++i) {
                mHitCount[i] = mContentCount[i] = 0;
            }
        }

        public static string GetCoverTag() {
            return CoverTag;
        }

        public static long GetContendCount(int syncId) {
            return mContentCount[syncId];
        }

        public static long GetHitCount(int syncId) {
            return mHitCount[syncId];
        }

        public static void MarkPoint(int syncId, bool didWait) {
            if (didWait) {
                Interlocked.Increment(ref mContentCount[syncId]);
            } else {
                Interlocked.Increment(ref mHitCount[syncId]);
            }
        }

        public static void Enter(object obj, int id) {
            bool didAquire = false;            
            MyMonitor.Enter(obj, ref didAquire, id);
        }

        public static void Enter(object obj, ref bool didAquire, int id) {
            didAquire = Monitor.TryEnter(obj);
            if (didAquire) {
                MarkPoint(id, false);
            } else {
                Monitor.Enter(obj);
                didAquire = true;
                MarkPoint(id, true);
            }
        }

        public static void Dump(string file) {
            using (StreamWriter w = new StreamWriter(file, true)) {
                w.WriteLine("~|B3|~");
                w.WriteLine("A|" + Assembly.GetEntryAssembly().Location);
                w.WriteLine("C|" + Environment.CommandLine);
                w.WriteLine("D|" + sTime.ToString("yyyyMMdd`HHmmss"));
                w.WriteLine("T|" + CoverTag);
                w.WriteLine("M|" + Environment.MachineName);
                w.WriteLine("O|" + Environment.OSVersion);
                w.WriteLine("P|" + Environment.ProcessorCount);
                w.WriteLine("SP|");

                for (int i = 0; i < mHitCount.Length; ++i) {
                    if (mHitCount[i] != 0 || mContentCount[i] != 0) {
                        w.WriteLine(i + "|" + mHitCount[i] + "|" + mContentCount[i]);
                    }
                }

                w.WriteLine("~|E|~");
            }
        }

    } // class

} // namespace
