﻿using System;
using System.IO;
using System.Diagnostics;
using System.Collections.Generic;
using System.Threading;
using System.Text.RegularExpressions;
using System.Collections.ObjectModel;

public class Cover {

    int coverPointId = 0;
    List<string> lines;
    readonly List<CoverData> data;

    int currentIndex;
    int lastLine = 0;
    bool inMethod = false;
    string lastFile = string.Empty;
    string methodName = string.Empty;
    string inClass = string.Empty;

    bool didInject = false;

    readonly Dictionary<string, string> magicMap = new Dictionary<string, string>() {

        {"void [mscorlib]System.Threading.Monitor::Enter(object)", "void System.Threading.MyMonitor::Enter(object, int32)"},
        {"void [mscorlib]System.Threading.Monitor::Enter(object, bool&)", "void System.Threading.MyMonitor::Enter(object, bool&, int32)"},

    };

    public Cover() {
        this.data = new List<CoverData>();
    }

    public ReadOnlyCollection<CoverData> Data {
        get {
            return data.AsReadOnly();
        }
    }

    protected void OnCall(string targetMethod) {
        string replaceString;

        if (lastFile.Contains("MyMonitor.cs")) return;

        if (magicMap.TryGetValue(targetMethod, out replaceString)) {
            didInject = true;

            lines[currentIndex] = lines[currentIndex].Replace(targetMethod, replaceString);

            lines.Insert(currentIndex, string.Format("ldc.i4 {0}", coverPointId));
            ++currentIndex;

            data.Add(new CoverData(coverPointId, lastLine, lastFile, inClass, methodName));
            ++coverPointId;

        } else if (lines[currentIndex].Contains("System.Threading.Monitor::Enter")) {
            // this is the list of methods variants we didn't cover.
            File.AppendAllText("oops.txt", string.Format("{0}|{2}|{1}", currentIndex, targetMethod, lastFile) + Environment.NewLine);
        }
    }

    private bool IsInClass(string classFullName) {
        return string.Equals(inClass, classFullName, StringComparison.CurrentCulture);
    }

    public void InstrumentAssembly(List<string> lines) {
        this.lines = lines;
        currentIndex = 0;
        lastLine = 0;
        inMethod = false;
        lastFile = string.Empty;
        methodName = string.Empty;
        inClass = string.Empty;

        int stackLine = -1;

        for (int i = 0; i < lines.Count; ++i) {
            if (inMethod) {
                if (lines[i].Contains("// end of method")) {
                    if (didInject) {
                        string x = lines[stackLine];
                        string[] parts = x.Split(new[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                        int y = int.Parse(parts[1]);
                        lines[stackLine] = string.Format(".maxstack {0}", y + 1);
                    }

                    didInject = false;
                    inMethod = false;
                    stackLine = -1;
                } else if (lines[i].Contains(".maxstack ")) {
                    stackLine = i;

                } else if (lines[i].Contains("call ")) {
                    currentIndex = i;

                    int j = 1;
                    while (lines[i].EndsWith(",")) {
                        lines[i] += " " + lines[i + j].TrimStart();
                        lines[i + j] = "/* MOVED UP!!! */";
                        j++;
                    }

                    int startIndex = lines[currentIndex].IndexOf("call ");
                    string targetMethod = lines[currentIndex].Substring(startIndex + 5).Trim();

                    OnCall(targetMethod);

                    i = currentIndex;

                } else if (lines[i].Contains("callvirt ")) {
                    currentIndex = i;

                    int j = 1;
                    while (lines[i].EndsWith(",")) {
                        lines[i] += " " + lines[i + j].TrimStart();
                        lines[i + j] = "/* MOVED UP!!! */";
                        j++;
                    }

                    int startIndex = lines[currentIndex].IndexOf("callvirt ");
                    string targetMethod = lines[currentIndex].Substring(startIndex + 9).Trim();

                    OnCall(targetMethod);

                    i = currentIndex;

                } else if (lines[i].Contains(".s ")) {
                    lines[i] = lines[i].Replace(".s ", " ");

#if true

                } else if (lines[i].Contains(" ble.s ")) {
                    lines[i] = lines[i].Replace("ble.s ", "ble ");

                } else if (lines[i].Contains("leave.s ")) {
                    lines[i] = lines[i].Replace("leave.s ", "leave ");

                } else if (lines[i].Contains("bne.un.s ")) {
                    lines[i] = lines[i].Replace("bne.un.s ", "bne.un ");

                } else if (lines[i].Contains("bge.s ")) {
                    lines[i] = lines[i].Replace("bge.s ", "bge ");

                } else if (lines[i].Contains("beq.s ")) {
                    lines[i] = lines[i].Replace("beq.s ", "beq ");

                } else if (lines[i].Contains("bgt.s")) {
                    lines[i] = lines[i].Replace("bgt.s", "bgt");

                } else if (lines[i].Contains("br.s")) {
                    lines[i] = lines[i].Replace("br.s", "br");

                } else if (lines[i].Contains("blt.un.s")) {
                    lines[i] = lines[i].Replace("blt.un.s", "blt.un");

                } else if (lines[i].Contains("brfalse.s")) {
                    lines[i] = lines[i].Replace("brfalse.s", "brfalse");

                } else if (lines[i].Contains("blt.s")) {
                    lines[i] = lines[i].Replace("blt.s", "blt");

                } else if (lines[i].Contains("brtrue.s")) {
                    lines[i] = lines[i].Replace("brtrue.s", "brtrue");
#endif
                } else if (lines[i].Contains(".line")) {
                    string l = lines[i];

                    int startIndex = l.IndexOf(',') + 1;
                    int endIndex = l.IndexOf(':', startIndex);

                    lastLine = int.Parse(l.Substring(startIndex, endIndex - startIndex));

                    startIndex = l.IndexOf('\'', endIndex) + 1;
                    endIndex = l.IndexOf('\'', startIndex);

                    if (endIndex > startIndex) {
                        lastFile = l.Substring(startIndex, endIndex - startIndex).Replace("\\\\", "\\");
                    }
                }

            } else if (lines[i].Contains(".class")) {
                int lastSpace = lines[i].LastIndexOf(' ');
                inClass = lines[i].Substring(lastSpace + 1);

            } else if (lines[i].Contains(".method")) {
                string decl = string.Empty;

                while (lines[i].Trim() != "{") {
                    decl += lines[i];
                    ++i;
                }

                int endIndex = decl.IndexOf('(');
                int startIndex = decl.LastIndexOf(' ', endIndex) + 1; // skip the space.

                inMethod = true;
                methodName = decl.Substring(startIndex, endIndex - startIndex);
            }
        }
    }

    internal void InjectMyMonitor(string coverTag, List<string> lines) {
        int idx = lines.FindIndex(a => a.Contains("// =============== CLASS MEMBERS DECLARATION ==================="));
        if (idx == -1) throw new Exception("Injection Point Changed!");

        Inject(coverTag, lines, idx, "SyncCover.il.myMonitor.txt");
    }

    internal void Inject(string coverTag, List<string> lines, int idx, params string[] resourceStreamNames) {
        System.Reflection.Assembly resourceAssembly = System.Reflection.Assembly.GetExecutingAssembly();

        List<string> newCode = new List<string>();

        foreach (string resourceStreamName in resourceStreamNames) {
            using (Stream srcStream = resourceAssembly.GetManifestResourceStream(resourceStreamName)) {
                newCode.InsertRange(newCode.Count, ReplaceString(coverTag, ReadAllLines(srcStream)));
            }
        }

        lines.InsertRange(idx + 1, newCode);
    }

    static IEnumerable<string> ReplaceString(string coverTag, IEnumerable<string> iEnumerable) {
        foreach (string s in iEnumerable) {
            yield return s.Replace("ldstr      \"CC_COVER_TAG_CC\"", "ldstr      \"" + coverTag + "\"");
        }
    }

    static IEnumerable<string> ReadAllLines(Stream srcStream) {
        using (StreamReader r = new StreamReader(srcStream)) {
            while (!r.EndOfStream) yield return r.ReadLine();
        }
    }

} // class
