﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

public class PeVerifyResult {

    public int ExitCode;
    public string AssemblyName;
    public List<string> Errors;

    readonly static Regex offsetEx = new Regex(@"\[offset 0x.*\]");

    public string NormalizeErrorString(string error) {
        // Lets remove any path information.
        string path = Path.GetDirectoryName(AssemblyName);

        StringBuilder b = new StringBuilder(error);
        if (path.Length > 0) {
            b.Replace(path, "<path>");
        }
        b.Replace("\r\n", " ");

        return offsetEx.Replace(b.ToString(), "[<offset/>]");
    }

} // class

public class PeVerify {

    const int PeVerifyExpectedExitCode = 0;
    
    readonly string peVerifyPathv3;

    public PeVerify(string peVerifyPathv3) {
        this.peVerifyPathv3 = peVerifyPathv3;
    }

    public PeVerifyResult VerifyAssembly(string assemblyName) {

        PeVerifyResult result = new PeVerifyResult();
        result.AssemblyName = assemblyName;

        string stdOut, stdErr;
        result.ExitCode = ProcessUtil.StartAndWaitForResult(
            peVerifyPathv3,
            assemblyName + " /IL /NOLOGO",
            out stdOut,
            out stdErr
        );

        ParseErrors(result, stdOut);

        return result;
    }

    void ParseErrors(PeVerifyResult result, string stdOut) {

        result.Errors = new List<string>();

        int startIndex = 0;
        while (startIndex < stdOut.Length) {
            startIndex = stdOut.IndexOf("[IL]:", startIndex);
            if (startIndex == -1) break;

            int endIndex = stdOut.IndexOf("[IL]:", startIndex + 1);
            if (endIndex == -1) {
                // Look for the last line...
                endIndex = stdOut.IndexOf("\r\n", startIndex + 1);
            }

            result.Errors.Add(result.NormalizeErrorString(stdOut.Substring(startIndex, endIndex - startIndex)));
            startIndex = endIndex;
        }
    }

    public static void Assert(PeVerifyResult expectedResult, PeVerifyResult actualResult) {
        foreach (var e in expectedResult.Errors) {
            actualResult.Errors.Remove(e);
        }

        // TODO: Ideally this will be with peverify v4.0, and this won't be needed.
        if (actualResult.Errors.Count > expectedResult.Errors.Count) {
            Console.WriteLine("Expected:");
            expectedResult.Errors.ForEach(Console.WriteLine);
            Console.WriteLine();

            Console.WriteLine("Actual:");
            actualResult.Errors.ForEach(Console.WriteLine);
            Console.WriteLine("PeVerify: {0}", expectedResult.AssemblyName);

            throw new Exception("PeVerify Failed with " + actualResult.Errors.Count + " different errors.");
        }
    }

} // class
