﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using SyncCover.Properties;

class Program {

    static int Main(string[] args) {

        var settings = Settings.Default;

        if (!File.Exists(settings.IlasmPath)) {
            throw new FileNotFoundException("ilasm", settings.IlasmPath);
        } else if (!File.Exists(settings.IldasmPath)) {
            throw new FileNotFoundException("ildasm", settings.IldasmPath);
        } else if (!File.Exists(settings.PeVerifyPath)) {
            throw new FileNotFoundException("peverify", settings.PeVerifyPath);
        } else if (args.Length != 2) {
            Console.WriteLine("Usage: <assemblyName> <versionTag>");
            return 1;
        }

        string assemblyPath = args[0];
        
        if (!Path.IsPathRooted(assemblyPath)) {
            assemblyPath = Path.Combine(Environment.CurrentDirectory, assemblyPath);
        }

        if (!File.Exists(assemblyPath)) {
            throw new Exception(
                string.Format("Can't find assembly to cover. [{0}]", assemblyPath)
            );
        }

        string versionTag = args[1];
        Console.WriteLine("assemblyPath:{0} versionTag:{1}", assemblyPath, versionTag);

        string libRoot = Path.GetDirectoryName(assemblyPath);
        string libPath = Path.Combine(libRoot, @"lib\");
        string outPath = Path.Combine(libRoot, @"out\");
        string mapFile = Path.Combine(outPath, string.Format("cc.map.{0}.txt", versionTag));

        if (!Directory.Exists(libPath)) {
            Directory.CreateDirectory(libPath);
        }

        if (!Directory.Exists(outPath)) {
            Directory.CreateDirectory(outPath);
        }

        Cover c = new Cover();
        var assembly = new Assembly(settings.IldasmPath, settings.IlasmPath, libPath, assemblyPath);

        Console.WriteLine("Instrumenting: " + assembly.VersionInfo.FileName);
        c.InstrumentAssembly(assembly.Lines);
        c.InjectMyMonitor(versionTag, assembly.Lines);

        assembly.WriteIL();
        assembly.Compile(new PeVerify(settings.PeVerifyPath), outPath);

        CoverMap.ToFile(mapFile, versionTag, c.Data);

        Console.WriteLine("mapFile: " + mapFile);
        Console.WriteLine("data.Count: " + c.Data.Count);

        return 0;
    }

} // class
