﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.IO;
using System.Windows.Data;

namespace SyncView {

    public class SourceCodeConverter : IValueConverter {

        Dictionary<string, ObservableCollection<LineInfo>> mData = new Dictionary<string, ObservableCollection<LineInfo>>();

        public object Convert(object value, System.Type targetType, object parameter, System.Globalization.CultureInfo culture) {
            ObservableCollection<LineInfo> c;
            if (!mData.TryGetValue((string)value, out c)) {
                mData[(string)value] = c = new ObservableCollection<LineInfo>(new List<LineInfo>(ReadFile((string)value)));
            }

            return c;
        }

        private IEnumerable<LineInfo> ReadFile(string fileName) {
            List<LineInfo> lines = new List<LineInfo>(File.ReadAllLines(fileName).Select((x, i) => new LineInfo() { Text = x, LineNumber = i + 1, Priority = 0 }));

            IEnumerable<CoverMapRow> cm = CoverMap.FromFile(Window1.MapFile);
            IEnumerable<CoverRun> coverData = CoverRun.FromFile(Window1.CoverFile);

            var coverage = coverData.SelectMany(cov => cov.Data);

            var cd = cm.Where(x => x.FileName == fileName).Select(syncpoint =>
                new {
                    SyncID = syncpoint.SyncId,
                    Line = syncpoint.LineNumber,
                    Entered = coverage.Where(cov => cov.SyncId == syncpoint.SyncId).Select(cov => cov.WasEntered).Contains(true),
                    Waited = coverage.Where(cov => cov.SyncId == syncpoint.SyncId).Select(cov => cov.DidWait).Contains(true)
                }
                );

            foreach (var x in cd) {
                lines[x.Line - 1].SyncId = x.SyncID;
                if (x.Waited) {
                    lines[x.Line - 1].Priority = 1;
                } else if (x.Entered) {
                    lines[x.Line - 1].Priority = 2;
                } else {
                    lines[x.Line - 1].Priority = 3;
                }
            }

            return lines;
        }

        public object ConvertBack(object value, System.Type targetType, object parameter, System.Globalization.CultureInfo culture) {
            throw new System.NotImplementedException();
        }

    } // class

} // namespace 

