﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;

namespace SyncView {

    public class LineData {
        public ObservableCollection<string> Files { get; set; }
    }

    public partial class Window1 : Window {

        LineData mData;

        public Window1() {
            InitializeComponent();

            mData = new LineData();
            mData.Files = new ObservableCollection<string>();

            DataContext = mData;
        }

        private void Button_NextClick(object sender, RoutedEventArgs e) {
            FindNext(1);
        }

        private void FindNext(int dir) {
            int i = lstSource.SelectedIndex > 0 ? lstSource.SelectedIndex + dir : 0;

            for ( ; i >= 0 && i < lstSource.Items.Count; i += dir) {
                LineInfo f = (LineInfo)lstSource.Items[i];
                if (f.Priority != 0) {
                    break;
                }
            }

            if (i >= 0 && i < lstSource.Items.Count) {
                lstSource.SelectedIndex = i;
                lstSource.ScrollIntoView(lstSource.SelectedItem);
            }
        }

        private void Button_PrevClick(object sender, RoutedEventArgs e) {
            FindNext(-1);
        }

        private void ListBox_SelectionChanged(object sender, System.Windows.Controls.SelectionChangedEventArgs e) {
        }

        private void lstSource_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e) {
            (((VisualTreeHelper.GetChild(lstSource, 0) as Border).Child) as ScrollViewer).ScrollToVerticalOffset(0);
        }

        private void Button_Click(object sender, RoutedEventArgs e) {

            CoverFile = GetCoverFile();
            MapFile = GetMapFile(CoverFile);

            // Process open file dialog box results
            if (File.Exists(CoverFile)) {

                foreach (var fileName in CoverMap.FromFile(MapFile).Select(x => x.FileName).Distinct()) {
                    if (File.Exists(fileName)) {
                        mData.Files.Add(fileName);
                    }
                }
            }

            DataContext = mData;
        }

        static string GetMapFile(string probePath) {
            var file = Directory.GetFiles(Path.GetDirectoryName(probePath), "cc.map.*.txt").FirstOrDefault();

            if (File.Exists(file)) return file;
            Microsoft.Win32.OpenFileDialog dlg;

            // Configure open file dialog box
            dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.InitialDirectory = Path.GetDirectoryName(probePath);
            dlg.Title = "Open Coverage Map File";
            dlg.FileName = "";
            dlg.DefaultExt = ".txt"; // Default file extension
            dlg.Filter = "Coverage Map(cc.cov.*.txt)|cc.map.*.txt";

            if (dlg.ShowDialog() == true) {
                return dlg.FileName;
            } else {
                return "";
            }
        }

        private static string GetCoverFile() {
            Microsoft.Win32.OpenFileDialog dlg;

            // Configure open file dialog box
            dlg = new Microsoft.Win32.OpenFileDialog();
            dlg.InitialDirectory = Environment.CurrentDirectory;
            dlg.Title = "Coverage Data File";
            dlg.FileName = "";
            dlg.DefaultExt = ".txt"; // Default file extension
            dlg.Filter = "Coverage Data(cc.cov.*.txt)|cc.cov.*.txt";

            if (dlg.ShowDialog() == true) {
                return dlg.FileName;
            } else {
                return "";
            }
        }

        // HACK: Pass this into the converters!
        internal static string MapFile; //  = "cc.map.DEMO.txt";
        internal static string CoverFile; //  = "cc.cov.FOO.DWEEZIL.txt";

    } // class

    public class LineInfo {

        public string Text { get; set; }
        public int LineNumber { get; set; }
        public int Priority { get; set; }
        public int SyncId { get; set; }

        public override string ToString() {
            return "MyFoo+ToString(TODO)";
        }

    } // class

} // namespace
